function _assertThisInitialized(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _getPrototypeOf(o) {
    _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf(o);
}
function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf(subClass, superClass);
}
function _objectSpread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty(target, key, source[key]);
        });
    }
    return target;
}
function _possibleConstructorReturn(self, call) {
    if (call && (_typeof(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized(self);
}
function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf(o, p);
}
var _typeof = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn(this, result);
    };
}
import EventEmitter from "@src/util/event.emitter";
import HighlightRange from "@src/model/range";
import HighlightSource from "@src/model/source";
import uuid from "@src/util/uuid";
import Hook from "@src/util/hook";
import getInteraction from "@src/util/interaction";
import Cache from "@src/data/cache";
import Painter from "@src/painter";
import { eventEmitter, getDefaultOptions, INTERNAL_ERROR_EVENT } from "@src/util/const";
import { ERROR, EventType, CreateFrom } from "@src/types";
import { addClass, removeClass, isHighlightWrapNode, getHighlightById, getExtraHighlightId, getHighlightsByRoot, getHighlightId, addEventListener, removeEventListener } from "@src/util/dom";
var Highlighter = /*#__PURE__*/ function(EventEmitter) {
    "use strict";
    _inherits(Highlighter, EventEmitter);
    var _super = _createSuper(Highlighter);
    function Highlighter(options) {
        _classCallCheck(this, Highlighter);
        var _this;
        _this = _super.call(this);
        _this.event = getInteraction();
        _this.run = function() {
            return addEventListener(_this.options.$root, _this.event.PointerEnd, _this._handleSelection);
        };
        _this.stop = function() {
            removeEventListener(_this.options.$root, _this.event.PointerEnd, _this._handleSelection);
        };
        _this.addClass = function(className, id) {
            _this.getDoms(id).forEach(function($n) {
                addClass($n, className);
            });
        };
        _this.removeClass = function(className, id) {
            _this.getDoms(id).forEach(function($n) {
                removeClass($n, className);
            });
        };
        _this.getIdByDom = function($node) {
            return getHighlightId($node, _this.options.$root);
        };
        _this.getExtraIdByDom = function($node) {
            return getExtraHighlightId($node, _this.options.$root);
        };
        _this.getDoms = function(id) {
            return id ? getHighlightById(_this.options.$root, id, _this.options.wrapTag) : getHighlightsByRoot(_this.options.$root, _this.options.wrapTag);
        };
        _this.dispose = function() {
            var $root = _this.options.$root;
            removeEventListener($root, _this.event.PointerOver, _this._handleHighlightHover);
            removeEventListener($root, _this.event.PointerEnd, _this._handleSelection);
            removeEventListener($root, _this.event.PointerTap, _this._handleHighlightClick);
            _this.removeAll();
        };
        _this.setOption = function(options) {
            _this.options = _objectSpread({}, _this.options, options);
            _this.painter = new Painter({
                $root: _this.options.$root,
                wrapTag: _this.options.wrapTag,
                className: _this.options.style.className,
                exceptSelectors: _this.options.exceptSelectors
            }, _this.hooks);
        };
        _this.fromRange = function(range) {
            var start = {
                $node: range.startContainer,
                offset: range.startOffset
            };
            var end = {
                $node: range.endContainer,
                offset: range.endOffset
            };
            var text = range.toString();
            var id = _this.hooks.Render.UUID.call(start, end, text);
            id = typeof id !== "undefined" && id !== null ? id : uuid();
            var hRange = new HighlightRange(start, end, text, id);
            if (!hRange) {
                eventEmitter.emit(INTERNAL_ERROR_EVENT, {
                    type: ERROR.RANGE_INVALID
                });
                return null;
            }
            return _this._highlightFromHRange(hRange);
        };
        _this.fromStore = function(start, end, text, id, extra) {
            var hs = new HighlightSource(start, end, text, id, extra);
            try {
                _this._highlightFromHSource(hs);
                return hs;
            } catch (err) {
                eventEmitter.emit(INTERNAL_ERROR_EVENT, {
                    type: ERROR.HIGHLIGHT_SOURCE_RECREATE,
                    error: err,
                    detail: hs
                });
                return null;
            }
        };
        _this._getHooks = function() {
            return {
                Render: {
                    UUID: new Hook("Render.UUID"),
                    SelectedNodes: new Hook("Render.SelectedNodes"),
                    WrapNode: new Hook("Render.WrapNode")
                },
                Serialize: {
                    Restore: new Hook("Serialize.Restore"),
                    RecordInfo: new Hook("Serialize.RecordInfo")
                },
                Remove: {
                    UpdateNodes: new Hook("Remove.UpdateNodes")
                }
            };
        };
        _this._highlightFromHRange = function(range) {
            var source = range.serialize(_this.options.$root, _this.hooks);
            var $wraps = _this.painter.highlightRange(range);
            if ($wraps.length === 0) {
                eventEmitter.emit(INTERNAL_ERROR_EVENT, {
                    type: ERROR.DOM_SELECTION_EMPTY
                });
                return null;
            }
            _this.cache.save(source);
            _this.emit(EventType.CREATE, {
                sources: [
                    source
                ],
                type: CreateFrom.INPUT
            }, _assertThisInitialized(_this));
            return source;
        };
        _this._handleSelection = function() {
            var range = HighlightRange.fromSelection(_this.options.$root, _this.hooks.Render.UUID);
            if (range) {
                _this._highlightFromHRange(range);
                HighlightRange.removeDomRange(_this.options.$root.ownerDocument.defaultView);
            }
        };
        _this._handleHighlightHover = function(e) {
            var $target = e.target;
            if (!isHighlightWrapNode($target)) {
                _this._hoverId && _this.emit(EventType.HOVER_OUT, {
                    id: _this._hoverId
                }, _assertThisInitialized(_this), e);
                _this._hoverId = null;
                return;
            }
            var id = getHighlightId($target, _this.options.$root);
            // prevent trigger in the same highlight range
            if (_this._hoverId === id) {
                return;
            }
            // hover another highlight range, need to trigger previous highlight hover out event
            if (_this._hoverId) {
                _this.emit(EventType.HOVER_OUT, {
                    id: _this._hoverId
                }, _assertThisInitialized(_this), e);
            }
            _this._hoverId = id;
            _this.emit(EventType.HOVER, {
                id: _this._hoverId
            }, _assertThisInitialized(_this), e);
        };
        _this._handleError = function(type) {
            if (_this.options.verbose) {
                // eslint-disable-next-line no-console
                console.warn(type);
            }
        };
        _this._handleHighlightClick = function(e) {
            var $target = e.target;
            if (isHighlightWrapNode($target)) {
                var id = getHighlightId($target, _this.options.$root);
                _this.emit(EventType.CLICK, {
                    id: id
                }, _assertThisInitialized(_this), e);
            }
        };
        _this.options = getDefaultOptions();
        // initialize hooks
        _this.hooks = _this._getHooks();
        _this.setOption(options);
        // initialize cache
        _this.cache = new Cache();
        var $root = _this.options.$root;
        // initialize event listener
        addEventListener($root, _this.event.PointerOver, _this._handleHighlightHover);
        // initialize event listener
        addEventListener($root, _this.event.PointerTap, _this._handleHighlightClick);
        eventEmitter.on(INTERNAL_ERROR_EVENT, _this._handleError);
        return _this;
    }
    var _proto = Highlighter.prototype;
    _proto.remove = function remove(id) {
        if (!id) {
            return;
        }
        var doseExist = this.painter.removeHighlight(id);
        this.cache.remove(id);
        // only emit REMOVE event when highlight exist
        if (doseExist) {
            this.emit(EventType.REMOVE, {
                ids: [
                    id
                ]
            }, this);
        }
    };
    _proto.removeAll = function removeAll() {
        this.painter.removeAllHighlight();
        var ids = this.cache.removeAll();
        this.emit(EventType.REMOVE, {
            ids: ids
        }, this);
    };
    _proto._highlightFromHSource = function _highlightFromHSource() {
        var sources = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : [];
        var renderedSources = this.painter.highlightSource(sources);
        this.emit(EventType.CREATE, {
            sources: renderedSources,
            type: CreateFrom.STORE
        }, this);
        this.cache.save(sources);
    };
    return Highlighter;
}(EventEmitter);
Highlighter.event = EventType;
Highlighter.isHighlightWrapNode = isHighlightWrapNode;
// eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
Highlighter.isHighlightSource = function(d) {
    return !!d.__isHighlightSource;
};
export { Highlighter as default };

//# sourceMappingURL=index.js.map