function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}
function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}
function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
import { SplitType, SelectedNodeType } from "@src/types";
import { hasClass, addClass as addElementClass, isHighlightWrapNode, removeAllClass } from "@src/util/dom";
import { ID_DIVISION, getDefaultOptions, CAMEL_DATASET_IDENTIFIER, CAMEL_DATASET_IDENTIFIER_EXTRA, DATASET_IDENTIFIER, DATASET_SPLIT_TYPE, DATASET_IDENTIFIER_EXTRA } from "../util/const";
import { unique } from "../util/tool";
/**
 * 支持的选择器类型
 *  - class: .title, .main-nav
 *  - id: #nav, #js-toggle-btn
 *  - tag: div, p, span
 */ var isMatchSelector = function($node, selector) {
    if (!$node) {
        return false;
    }
    if (/^\./.test(selector)) {
        var className = selector.replace(/^\./, "");
        return $node && hasClass($node, className);
    } else if (/^#/.test(selector)) {
        var id = selector.replace(/^#/, "");
        return $node && $node.id === id;
    }
    var tagName = selector.toUpperCase();
    return $node && $node.tagName === tagName;
};
/**
 * If start node and end node is the same, don't need to tranvers the dom tree.
 */ var getNodesIfSameStartEnd = function($startNode, startOffset, endOffset, exceptSelectors) {
    var $element = $startNode;
    var isExcepted = function($e) {
        return exceptSelectors === null || exceptSelectors === void 0 ? void 0 : exceptSelectors.some(function(s) {
            return isMatchSelector($e, s);
        });
    };
    while($element){
        if ($element.nodeType === 1 && isExcepted($element)) {
            return [];
        }
        $element = $element.parentNode;
    }
    $startNode.splitText(startOffset);
    var passedNode = $startNode.nextSibling;
    passedNode.splitText(endOffset - startOffset);
    return [
        {
            $node: passedNode,
            type: SelectedNodeType.text,
            splitType: SplitType.both
        }
    ];
};
/**
 * get all the dom nodes between the start and end node
 */ export var getSelectedNodes = function($root, start, end, exceptSelectors) {
    var $startNode = start.$node;
    var $endNode = end.$node;
    var startOffset = start.offset;
    var endOffset = end.offset;
    // split current node when the start-node and end-node is the same
    if ($startNode === $endNode && $startNode.nodeType === Node.TEXT_NODE) {
        return getNodesIfSameStartEnd($startNode, startOffset, endOffset, exceptSelectors);
    }
    var nodeStack = [
        $root
    ];
    var selectedNodes = [];
    var isExcepted = function($e) {
        return exceptSelectors === null || exceptSelectors === void 0 ? void 0 : exceptSelectors.some(function(s) {
            return isMatchSelector($e, s);
        });
    };
    var withinSelectedRange = false;
    var curNode = null;
    while(curNode = nodeStack.pop()){
        // do not traverse the excepted node
        if (curNode.nodeType === 1 && isExcepted(curNode)) {
            continue;
        }
        var children = curNode.childNodes;
        for(var i = children.length - 1; i >= 0; i--){
            nodeStack.push(children[i]);
        }
        // only collect text nodes
        if (curNode === $startNode) {
            if (curNode.nodeType === 3) {
                curNode.splitText(startOffset);
                var node = curNode.nextSibling;
                selectedNodes.push({
                    $node: node,
                    type: SelectedNodeType.text,
                    splitType: SplitType.head
                });
            }
            // meet the start-node (begin to traverse)
            withinSelectedRange = true;
        } else if (curNode === $endNode) {
            if (curNode.nodeType === 3) {
                var node1 = curNode;
                node1.splitText(endOffset);
                selectedNodes.push({
                    $node: node1,
                    type: SelectedNodeType.text,
                    splitType: SplitType.tail
                });
            }
            break;
        } else if (withinSelectedRange && curNode.nodeType === 3) {
            selectedNodes.push({
                $node: curNode,
                type: SelectedNodeType.text,
                splitType: SplitType.none
            });
        }
    }
    return selectedNodes;
};
var addClass = function($el, className) {
    var classNames = Array.isArray(className) ? className : [
        className
    ];
    classNames = classNames.length === 0 ? [
        getDefaultOptions().style.className
    ] : classNames;
    classNames.forEach(function(c) {
        addElementClass($el, c);
    });
    return $el;
};
var isNodeEmpty = function($n) {
    return !$n || !$n.textContent;
};
/**
 * Wrap a common wrapper.
 */ var wrapNewNode = function(selected, range, className, wrapTag) {
    var $wrap = document.createElement(wrapTag);
    addClass($wrap, className);
    $wrap.appendChild(selected.$node.cloneNode(false));
    selected.$node.parentNode.replaceChild($wrap, selected.$node);
    $wrap.setAttribute("data-".concat(DATASET_IDENTIFIER), range.id);
    $wrap.setAttribute("data-".concat(DATASET_SPLIT_TYPE), selected.splitType);
    $wrap.setAttribute("data-".concat(DATASET_IDENTIFIER_EXTRA), "");
    return $wrap;
};
/**
 * Split and wrapper each one.
 */ var wrapPartialNode = function(selected, range, className, wrapTag) {
    var $wrap = document.createElement(wrapTag);
    var $parent = selected.$node.parentNode;
    var $prev = selected.$node.previousSibling;
    var $next = selected.$node.nextSibling;
    var $fr = document.createDocumentFragment();
    var parentId = $parent.dataset[CAMEL_DATASET_IDENTIFIER];
    var parentExtraId = $parent.dataset[CAMEL_DATASET_IDENTIFIER_EXTRA];
    var extraInfo = parentExtraId ? parentId + ID_DIVISION + parentExtraId : parentId;
    $wrap.setAttribute("data-".concat(DATASET_IDENTIFIER), range.id);
    $wrap.setAttribute("data-".concat(DATASET_IDENTIFIER_EXTRA), extraInfo);
    $wrap.appendChild(selected.$node.cloneNode(false));
    var headSplit = false;
    var tailSplit = false;
    var splitType;
    if ($prev) {
        var $span = $parent.cloneNode(false);
        $span.textContent = $prev.textContent;
        $fr.appendChild($span);
        headSplit = true;
    }
    var classNameList = [];
    if (Array.isArray(className)) {
        var _classNameList;
        (_classNameList = classNameList).push.apply(_classNameList, _toConsumableArray(className));
    } else {
        classNameList.push(className);
    }
    addClass($wrap, unique(classNameList));
    $fr.appendChild($wrap);
    if ($next) {
        var $span1 = $parent.cloneNode(false);
        $span1.textContent = $next.textContent;
        $fr.appendChild($span1);
        tailSplit = true;
    }
    if (headSplit && tailSplit) {
        splitType = SplitType.both;
    } else if (headSplit) {
        splitType = SplitType.head;
    } else if (tailSplit) {
        splitType = SplitType.tail;
    } else {
        splitType = SplitType.none;
    }
    $wrap.setAttribute("data-".concat(DATASET_SPLIT_TYPE), splitType);
    $parent.parentNode.replaceChild($fr, $parent);
    return $wrap;
};
/**
 * Just update id info (no wrapper updated).
 */ var wrapOverlapNode = function(selected, range, className) {
    var $parent = selected.$node.parentNode;
    var $wrap = $parent;
    removeAllClass($wrap);
    addClass($wrap, className);
    var dataset = $parent.dataset;
    var formerId = dataset[CAMEL_DATASET_IDENTIFIER];
    dataset[CAMEL_DATASET_IDENTIFIER] = range.id;
    dataset[CAMEL_DATASET_IDENTIFIER_EXTRA] = dataset[CAMEL_DATASET_IDENTIFIER_EXTRA] ? formerId + ID_DIVISION + dataset[CAMEL_DATASET_IDENTIFIER_EXTRA] : formerId;
    return $wrap;
};
/**
 * wrap a dom node with highlight wrapper
 *
 * Because of supporting the highlight-overlapping,
 * Highlighter can't just wrap all nodes in a simple way.
 * There are three types:
 *  - wrapping a whole new node (without any wrapper)
 *  - wrapping part of the node
 *  - wrapping the whole wrapped node
 */ export var wrapHighlight = function(selected, range, className, wrapTag) {
    var $parent = selected.$node.parentNode;
    var $prev = selected.$node.previousSibling;
    var $next = selected.$node.nextSibling;
    var $wrap;
    // text node, not in a highlight wrapper -> should be wrapped in a highlight wrapper
    if (!isHighlightWrapNode($parent)) {
        $wrap = wrapNewNode(selected, range, className, wrapTag);
    } else if (isHighlightWrapNode($parent) && (!isNodeEmpty($prev) || !isNodeEmpty($next))) {
        $wrap = wrapPartialNode(selected, range, className, wrapTag);
    } else {
        $wrap = wrapOverlapNode(selected, range, className);
    }
    return $wrap;
};
/**
 * merge the adjacent text nodes
 * .normalize() API has some bugs in IE11
 */ export var normalizeSiblingText = function($s) {
    var isNext = arguments.length > 1 && arguments[1] !== void 0 ? arguments[1] : true;
    if (!$s || $s.nodeType !== 3) {
        return;
    }
    var $sibling = isNext ? $s.nextSibling : $s.previousSibling;
    if ($sibling.nodeType !== 3) {
        return;
    }
    var text = $sibling.nodeValue;
    $s.nodeValue = isNext ? $s.nodeValue + text : text + $s.nodeValue;
    $sibling.parentNode.removeChild($sibling);
};

//# sourceMappingURL=dom.js.map