/**
 * Painter object is designed for some painting work about higlighting,
 * including rendering, cleaning...
 * No need to instantiate repeatly. A Highlighter instance will bind a Painter instance.
 */ function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}
function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _instanceof(left, right) {
    if (right != null && typeof Symbol !== "undefined" && right[Symbol.hasInstance]) {
        return !!right[Symbol.hasInstance](left);
    } else {
        return left instanceof right;
    }
}
function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}
function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
import HighlightSource from "@src/model/source";
import { wrapHighlight, getSelectedNodes, normalizeSiblingText } from "@src/painter/dom";
import { getHighlightsByRoot, forEach, addClass, removeAllClass } from "@src/util/dom";
import { ERROR } from "@src/types";
import { initDefaultStylesheet } from "@src/painter/style";
import { ID_DIVISION, eventEmitter, DATASET_IDENTIFIER, INTERNAL_ERROR_EVENT, CAMEL_DATASET_IDENTIFIER, CAMEL_DATASET_IDENTIFIER_EXTRA } from "@src/util/const";
var Painter = /*#__PURE__*/ function() {
    "use strict";
    function Painter(options, hooks) {
        _classCallCheck(this, Painter);
        this.options = {
            $root: options.$root,
            wrapTag: options.wrapTag,
            exceptSelectors: options.exceptSelectors,
            className: options.className
        };
        this.hooks = hooks;
        initDefaultStylesheet();
    }
    var _proto = Painter.prototype;
    /* =========================== render =========================== */ _proto.highlightRange = function highlightRange(range) {
        var _this = this;
        if (!range.frozen) {
            throw ERROR.HIGHLIGHT_RANGE_FROZEN;
        }
        var _this_options = this.options, $root = _this_options.$root, className = _this_options.className, exceptSelectors = _this_options.exceptSelectors;
        var hooks = this.hooks;
        var $selectedNodes = getSelectedNodes($root, range.start, range.end, exceptSelectors);
        if (!hooks.Render.SelectedNodes.isEmpty()) {
            $selectedNodes = hooks.Render.SelectedNodes.call(range.id, $selectedNodes) || [];
        }
        return $selectedNodes.map(function(n) {
            var $node = wrapHighlight(n, range, className, _this.options.wrapTag);
            if (!hooks.Render.WrapNode.isEmpty()) {
                $node = hooks.Render.WrapNode.call(range.id, $node);
            }
            return $node;
        });
    };
    _proto.highlightSource = function highlightSource(sources) {
        var _this = this;
        var list = Array.isArray(sources) ? sources : [
            sources
        ];
        var renderedSources = [];
        list.forEach(function(s) {
            if (!_instanceof(s, HighlightSource)) {
                eventEmitter.emit(INTERNAL_ERROR_EVENT, {
                    type: ERROR.SOURCE_TYPE_ERROR
                });
                return;
            }
            var range = s.deSerialize(_this.options.$root, _this.hooks);
            var $nodes = _this.highlightRange(range);
            if ($nodes.length > 0) {
                renderedSources.push(s);
            } else {
                eventEmitter.emit(INTERNAL_ERROR_EVENT, {
                    type: ERROR.HIGHLIGHT_SOURCE_NONE_RENDER,
                    detail: s
                });
            }
        });
        return renderedSources;
    };
    /* ============================================================== */ /* =========================== clean =========================== */ // id: target id - highlight with this id should be clean
    // if there is no highlight for this id, it will return false, vice versa
    _proto.removeHighlight = function removeHighlight(id) {
        // whether extra ids contains the target id
        var reg = new RegExp("(".concat(id, "\\").concat(ID_DIVISION, "|\\").concat(ID_DIVISION, "?").concat(id, "$)"));
        var hooks = this.hooks;
        var wrapTag = this.options.wrapTag;
        var $spans = document.querySelectorAll("".concat(wrapTag, "[data-").concat(DATASET_IDENTIFIER, "]"));
        // nodes to remove
        var $toRemove = [];
        // nodes to update main id
        var $idToUpdate = [];
        // nodes to update extra id
        var $extraToUpdate = [];
        var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
        try {
            for(var _iterator = $spans[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
                var $s = _step.value;
                var spanId = $s.dataset[CAMEL_DATASET_IDENTIFIER];
                var spanExtraIds = $s.dataset[CAMEL_DATASET_IDENTIFIER_EXTRA];
                // main id is the target id and no extra ids --> to remove
                if (spanId === id && !spanExtraIds) {
                    $toRemove.push($s);
                } else if (spanId === id) {
                    $idToUpdate.push($s);
                } else if (spanId !== id && reg.test(spanExtraIds)) {
                    $extraToUpdate.push($s);
                }
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally{
            try {
                if (!_iteratorNormalCompletion && _iterator.return != null) {
                    _iterator.return();
                }
            } finally{
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }
        $toRemove.forEach(function($s) {
            var $parent = $s.parentNode;
            var $fr = document.createDocumentFragment();
            forEach($s.childNodes, function($c) {
                return $fr.appendChild($c.cloneNode(false));
            });
            var $prev = $s.previousSibling;
            var $next = $s.nextSibling;
            $parent.replaceChild($fr, $s);
            // there are bugs in IE11, so use a more reliable function
            normalizeSiblingText($prev, true);
            normalizeSiblingText($next, false);
            hooks.Remove.UpdateNodes.call(id, $s, "remove");
        });
        $idToUpdate.forEach(function($s) {
            var dataset = $s.dataset;
            var ids = dataset[CAMEL_DATASET_IDENTIFIER_EXTRA].split(ID_DIVISION);
            var newId = ids.shift();
            // find overlapped wrapper associated with "extra id"
            var $overlapSpan = document.querySelector("".concat(wrapTag, "[data-").concat(DATASET_IDENTIFIER, '="').concat(newId, '"]'));
            if ($overlapSpan) {
                // empty the current class list
                removeAllClass($s);
                // retain the class list of the overlapped wrapper which associated with "extra id"
                addClass($s, _toConsumableArray($overlapSpan.classList));
            }
            dataset[CAMEL_DATASET_IDENTIFIER] = newId;
            dataset[CAMEL_DATASET_IDENTIFIER_EXTRA] = ids.join(ID_DIVISION);
            hooks.Remove.UpdateNodes.call(id, $s, "id-update");
        });
        $extraToUpdate.forEach(function($s) {
            var extraIds = $s.dataset[CAMEL_DATASET_IDENTIFIER_EXTRA];
            $s.dataset[CAMEL_DATASET_IDENTIFIER_EXTRA] = extraIds.replace(reg, "");
            hooks.Remove.UpdateNodes.call(id, $s, "extra-update");
        });
        return $toRemove.length + $idToUpdate.length + $extraToUpdate.length !== 0;
    };
    _proto.removeAllHighlight = function removeAllHighlight() {
        var _this_options = this.options, wrapTag = _this_options.wrapTag, $root = _this_options.$root;
        var $spans = getHighlightsByRoot($root, wrapTag);
        $spans.forEach(function($s) {
            var $parent = $s.parentNode;
            var $fr = document.createDocumentFragment();
            forEach($s.childNodes, function($c) {
                return $fr.appendChild($c.cloneNode(false));
            });
            $parent.replaceChild($fr, $s);
        });
    };
    return Painter;
}();
export { Painter as default };

//# sourceMappingURL=index.js.map