function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}
function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}
function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
import { ID_DIVISION, DATASET_IDENTIFIER, CAMEL_DATASET_IDENTIFIER, CAMEL_DATASET_IDENTIFIER_EXTRA } from "@src/util/const";
/**
 * whether a wrapper node
 */ export var isHighlightWrapNode = function($node) {
    return !!$node.dataset && !!$node.dataset[CAMEL_DATASET_IDENTIFIER];
};
/**
 * ===================================================================================
 * below methods (getHighlightId/getExtraHighlightId)
 * will check whether the node is inside a wrapper iteratively util reach the root node
 * if the node is not inside the root, the id must be empty
 * ====================================================================================
 */ var findAncestorWrapperInRoot = function($node, $root) {
    var isInsideRoot = false;
    var $wrapper = null;
    while($node){
        if (isHighlightWrapNode($node)) {
            $wrapper = $node;
        }
        if ($node === $root) {
            isInsideRoot = true;
            break;
        }
        $node = $node.parentNode;
    }
    return isInsideRoot ? $wrapper : null;
};
/**
 * get highlight id by a node
 */ export var getHighlightId = function($node, $root) {
    $node = findAncestorWrapperInRoot($node, $root);
    if (!$node) {
        return "";
    }
    return $node.dataset[CAMEL_DATASET_IDENTIFIER];
};
/**
 * get extra highlight id by a node
 */ export var getExtraHighlightId = function($node, $root) {
    $node = findAncestorWrapperInRoot($node, $root);
    if (!$node) {
        return [];
    }
    return $node.dataset[CAMEL_DATASET_IDENTIFIER_EXTRA].split(ID_DIVISION).filter(function(i) {
        return i;
    });
};
/**
 * get all highlight wrapping nodes nodes from a root node
 */ export var getHighlightsByRoot = function($roots, wrapTag) {
    if (!Array.isArray($roots)) {
        $roots = [
            $roots
        ];
    }
    var $wraps = [];
    var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
    try {
        for(var _iterator = $roots[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
            var $r = _step.value;
            var $list = $r.querySelectorAll("".concat(wrapTag, "[data-").concat(DATASET_IDENTIFIER, "]"));
            // eslint-disable-next-line prefer-spread
            $wraps.push.apply($wraps, $list);
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally{
        try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
                _iterator.return();
            }
        } finally{
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
    return $wraps;
};
/**
 * get all highlight wrapping nodes by highlight id from a root node
 */ export var getHighlightById = function($root, id, wrapTag) {
    var $highlights = [];
    var reg = new RegExp("(".concat(id, "\\").concat(ID_DIVISION, "|\\").concat(ID_DIVISION, "?").concat(id, "$)"));
    var $list = $root.querySelectorAll("".concat(wrapTag, "[data-").concat(DATASET_IDENTIFIER, "]"));
    var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = undefined;
    try {
        for(var _iterator = $list[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true){
            var $l = _step.value;
            var $n = $l;
            var nid = $n.dataset[CAMEL_DATASET_IDENTIFIER];
            if (nid === id) {
                $highlights.push($n);
                continue;
            }
            var extraId = $n.dataset[CAMEL_DATASET_IDENTIFIER_EXTRA];
            if (reg.test(extraId)) {
                $highlights.push($n);
                continue;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally{
        try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
                _iterator.return();
            }
        } finally{
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }
    return $highlights;
};
export var forEach = function($nodes, cb) {
    for(var i = 0; i < $nodes.length; i++){
        cb($nodes[i], i, $nodes);
    }
};
export var removeEventListener = function($el, evt, fn) {
    $el.removeEventListener(evt, fn);
};
/**
 * maybe be need some polyfill methods later
 * provide unified dom methods for compatibility
 */ export var addEventListener = function($el, evt, fn) {
    $el.addEventListener(evt, fn);
    return function() {
        removeEventListener($el, evt, fn);
    };
};
export var addClass = function($el, className) {
    var _$el_classList;
    if (!Array.isArray(className)) {
        className = [
            className
        ];
    }
    (_$el_classList = $el.classList).add.apply(_$el_classList, _toConsumableArray(className));
};
export var removeClass = function($el, className) {
    $el.classList.remove(className);
};
export var removeAllClass = function($el) {
    $el.className = "";
};
export var hasClass = function($el, className) {
    return $el.classList.contains(className);
};

//# sourceMappingURL=dom.js.map