"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
exports.__esModule = true;
var event_emitter_1 = require("@src/util/event.emitter");
var range_1 = require("@src/model/range");
var source_1 = require("@src/model/source");
var uuid_1 = require("@src/util/uuid");
var hook_1 = require("@src/util/hook");
var interaction_1 = require("@src/util/interaction");
var cache_1 = require("@src/data/cache");
var painter_1 = require("@src/painter");
var const_1 = require("@src/util/const");
var types_1 = require("@src/types");
var dom_1 = require("@src/util/dom");
var Highlighter = /** @class */ (function (_super) {
    __extends(Highlighter, _super);
    function Highlighter(options) {
        var _this = _super.call(this) || this;
        _this.event = (0, interaction_1["default"])();
        _this.run = function () { return (0, dom_1.addEventListener)(_this.options.$root, _this.event.PointerEnd, _this._handleSelection); };
        _this.stop = function () {
            (0, dom_1.removeEventListener)(_this.options.$root, _this.event.PointerEnd, _this._handleSelection);
        };
        _this.addClass = function (className, id) {
            _this.getDoms(id).forEach(function ($n) {
                (0, dom_1.addClass)($n, className);
            });
        };
        _this.removeClass = function (className, id) {
            _this.getDoms(id).forEach(function ($n) {
                (0, dom_1.removeClass)($n, className);
            });
        };
        _this.getIdByDom = function ($node) { return (0, dom_1.getHighlightId)($node, _this.options.$root); };
        _this.getExtraIdByDom = function ($node) { return (0, dom_1.getExtraHighlightId)($node, _this.options.$root); };
        _this.getDoms = function (id) {
            return id
                ? (0, dom_1.getHighlightById)(_this.options.$root, id, _this.options.wrapTag)
                : (0, dom_1.getHighlightsByRoot)(_this.options.$root, _this.options.wrapTag);
        };
        _this.dispose = function () {
            var $root = _this.options.$root;
            (0, dom_1.removeEventListener)($root, _this.event.PointerOver, _this._handleHighlightHover);
            (0, dom_1.removeEventListener)($root, _this.event.PointerEnd, _this._handleSelection);
            (0, dom_1.removeEventListener)($root, _this.event.PointerTap, _this._handleHighlightClick);
            _this.removeAll();
        };
        _this.setOption = function (options) {
            _this.options = __assign(__assign({}, _this.options), options);
            _this.painter = new painter_1["default"]({
                $root: _this.options.$root,
                wrapTag: _this.options.wrapTag,
                className: _this.options.style.className,
                exceptSelectors: _this.options.exceptSelectors
            }, _this.hooks);
        };
        _this.fromRange = function (range) {
            var start = {
                $node: range.startContainer,
                offset: range.startOffset
            };
            var end = {
                $node: range.endContainer,
                offset: range.endOffset
            };
            var text = range.toString();
            var id = _this.hooks.Render.UUID.call(start, end, text);
            id = typeof id !== 'undefined' && id !== null ? id : (0, uuid_1["default"])();
            var hRange = new range_1["default"](start, end, text, id);
            if (!hRange) {
                const_1.eventEmitter.emit(const_1.INTERNAL_ERROR_EVENT, {
                    type: types_1.ERROR.RANGE_INVALID
                });
                return null;
            }
            return _this._highlightFromHRange(hRange);
        };
        _this.fromStore = function (start, end, text, id, extra) {
            var hs = new source_1["default"](start, end, text, id, extra);
            try {
                _this._highlightFromHSource(hs);
                return hs;
            }
            catch (err) {
                const_1.eventEmitter.emit(const_1.INTERNAL_ERROR_EVENT, {
                    type: types_1.ERROR.HIGHLIGHT_SOURCE_RECREATE,
                    error: err,
                    detail: hs
                });
                return null;
            }
        };
        _this._getHooks = function () { return ({
            Render: {
                UUID: new hook_1["default"]('Render.UUID'),
                SelectedNodes: new hook_1["default"]('Render.SelectedNodes'),
                WrapNode: new hook_1["default"]('Render.WrapNode')
            },
            Serialize: {
                Restore: new hook_1["default"]('Serialize.Restore'),
                RecordInfo: new hook_1["default"]('Serialize.RecordInfo')
            },
            Remove: {
                UpdateNodes: new hook_1["default"]('Remove.UpdateNodes')
            }
        }); };
        _this._highlightFromHRange = function (range) {
            var source = range.serialize(_this.options.$root, _this.hooks);
            var $wraps = _this.painter.highlightRange(range);
            if ($wraps.length === 0) {
                const_1.eventEmitter.emit(const_1.INTERNAL_ERROR_EVENT, {
                    type: types_1.ERROR.DOM_SELECTION_EMPTY
                });
                return null;
            }
            _this.cache.save(source);
            _this.emit(types_1.EventType.CREATE, { sources: [source], type: types_1.CreateFrom.INPUT }, _this);
            return source;
        };
        _this._handleSelection = function () {
            var range = range_1["default"].fromSelection(_this.hooks.Render.UUID);
            if (range) {
                _this._highlightFromHRange(range);
                range_1["default"].removeDomRange();
            }
        };
        _this._handleHighlightHover = function (e) {
            var $target = e.target;
            if (!(0, dom_1.isHighlightWrapNode)($target)) {
                _this._hoverId && _this.emit(types_1.EventType.HOVER_OUT, { id: _this._hoverId }, _this, e);
                _this._hoverId = null;
                return;
            }
            var id = (0, dom_1.getHighlightId)($target, _this.options.$root);
            // prevent trigger in the same highlight range
            if (_this._hoverId === id) {
                return;
            }
            // hover another highlight range, need to trigger previous highlight hover out event
            if (_this._hoverId) {
                _this.emit(types_1.EventType.HOVER_OUT, { id: _this._hoverId }, _this, e);
            }
            _this._hoverId = id;
            _this.emit(types_1.EventType.HOVER, { id: _this._hoverId }, _this, e);
        };
        _this._handleError = function (type) {
            if (_this.options.verbose) {
                // eslint-disable-next-line no-console
                console.warn(type);
            }
        };
        _this._handleHighlightClick = function (e) {
            var $target = e.target;
            if ((0, dom_1.isHighlightWrapNode)($target)) {
                var id = (0, dom_1.getHighlightId)($target, _this.options.$root);
                _this.emit(types_1.EventType.CLICK, { id: id }, _this, e);
            }
        };
        _this.options = (0, const_1.getDefaultOptions)();
        // initialize hooks
        _this.hooks = _this._getHooks();
        _this.setOption(options);
        // initialize cache
        _this.cache = new cache_1["default"]();
        var $root = _this.options.$root;
        // initialize event listener
        (0, dom_1.addEventListener)($root, _this.event.PointerOver, _this._handleHighlightHover);
        // initialize event listener
        (0, dom_1.addEventListener)($root, _this.event.PointerTap, _this._handleHighlightClick);
        const_1.eventEmitter.on(const_1.INTERNAL_ERROR_EVENT, _this._handleError);
        return _this;
    }
    Highlighter.prototype.remove = function (id) {
        if (!id) {
            return;
        }
        var doseExist = this.painter.removeHighlight(id);
        this.cache.remove(id);
        // only emit REMOVE event when highlight exist
        if (doseExist) {
            this.emit(types_1.EventType.REMOVE, { ids: [id] }, this);
        }
    };
    Highlighter.prototype.removeAll = function () {
        this.painter.removeAllHighlight();
        var ids = this.cache.removeAll();
        this.emit(types_1.EventType.REMOVE, { ids: ids }, this);
    };
    Highlighter.prototype._highlightFromHSource = function (sources) {
        if (sources === void 0) { sources = []; }
        var renderedSources = this.painter.highlightSource(sources);
        this.emit(types_1.EventType.CREATE, { sources: renderedSources, type: types_1.CreateFrom.STORE }, this);
        this.cache.save(sources);
    };
    Highlighter.event = types_1.EventType;
    Highlighter.isHighlightWrapNode = dom_1.isHighlightWrapNode;
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    Highlighter.isHighlightSource = function (d) { return !!d.__isHighlightSource; };
    return Highlighter;
}(event_emitter_1["default"]));
exports["default"] = Highlighter;
